'''
To evaluate Task 1B of CL eYRC23-24
Name: Jaison J.
Credit: e-Yantra, IIT Bombay
CDate: 12-09-2023
'''

import rclpy
import shutil
from rclpy.node import Node
from rclpy.serialization import serialize_message
import rosbag2_py
import tf2_ros
from rich.console import Console

class SimpleBagRecorder(Node):
    def __init__(self):
        super().__init__('simple_bag_recorder')
        self.tf_buffer = tf2_ros.buffer.Buffer()
        self.r = tf2_ros.TransformBroadcaster(self)
        self.listener = tf2_ros.TransformListener(self.tf_buffer, self)
        self.writer = rosbag2_py.SequentialWriter()
        try:
            shutil.rmtree("my_bag")
        except:
            pass
        storage_options = rosbag2_py._storage.StorageOptions(
            uri='my_bag',
            storage_id='sqlite3')
        converter_options = rosbag2_py._storage.ConverterOptions('', '')
        self.writer.open(storage_options, converter_options)
        topic_info = rosbag2_py._storage.TopicMetadata(
            name='tf',
            type= 'geometry_msgs/msg/TransformStamped', #'std_msgs/msg/String',
            serialization_format='cdr')
        self.writer.create_topic(topic_info)
        self.create_timer(0.1, self.topic_callback)

    def topic_callback(self):
        try:
            self.tool_link = self.tf_buffer.lookup_transform('base_link', "tool0", rclpy.time.Time())
            # print(type(self.tool_link))
            self.get_logger().info("\033[92m DATA COLLECTION STARTED \033[00m", once=True)
            self.writer.write('tf', serialize_message(self.tool_link), self.get_clock().now().nanoseconds)
        except Exception as e:
            self.get_logger().warning("WAITING FOR DATA", once=True)
            pass


def evaluate():
    result = {}
    console = Console()
    rclpy.init()
    sbr = SimpleBagRecorder()
    try:
        rclpy.spin(sbr)
    except KeyboardInterrupt:
        console.print(f"[green]\nExiting Gracefully[/green]")
    # rclpy.shutdown()
    result["generate"] = False
    return result


# standard imports
import platform
import subprocess
import sys
import os

# third-party imprts
import distro
from eyantra_autoeval.utils.common import is_docker, run_shell_command
from rich import print
from rich.console import Console
from rich.prompt import Confirm

                            

cb_count = 0
seen_frames = set()

def evaluate():

    result = {}
    console = Console()
    
    with console.status("[bold green]Welcome to ROS2 MOOC...") as status:
        console.print(f"[bold green]Welcome to ROS2 MOOC...🤖[/bold green]")

    with console.status("[bold green]Gathering data🪄...") as status:

        console.print(f"[green]Gathering system information🪄[/green]")
        distribution = {
            "machine": platform.machine(),
            "release": platform.release(),
            "system": platform.system(),
            "version": distro.version(best=True),
            "name": distro.name(),
        }
        result["distro"] = distribution
        result["virtualized"] = "hypervisor" in run_shell_command("cat /proc/cpuinfo")
        result["dockerized"] = is_docker()

        self_os_declaration = False
    if (result["distro"]["name"] != "Ubuntu"
        or "22.04" not in result["distro"]["version"]
        or result["virtualized"]
        or result["dockerized"]):
        console.print(
            f"""You seem to be using [bold][blue]{result["distro"]["name"]} {result["distro"]["version"]} ({result["distro"]["machine"]}) on {"virtualized" if result["virtualized"] else ""}{"dockerized" if result["dockerized"] else ""} environment[/bold][/blue]. We only support [bold][blue]Ubuntu 22.04 on (x86_64) on baremetal[/bold][/blue]. You may continue to use the existing setup, and you shall not be penalized for the same. However, we’ll officially only support Ubuntu 22.04 as the OS of choice, and we won’t be to help you out with any installation related questions or package version incompatibility issues🙂."""
        )
    result["ros"] = {}


    with console.status("[bold green]Gathering data🪄...") as status:

        console.print(f"[green]Gathering information on python🪄[/green]")
        result["python"] = {}
        result["python"][
            "version"
        ] = f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"

        console.print(f"[green]Gathering information on ROS distribution🪄[/green]")
        
        result["ros"]["version"] = (
            subprocess.run(
                "source /opt/ros/humble/setup.bash && printenv ROS_DISTRO",
                capture_output=True,
                shell=True,
                executable="/bin/bash", 
            )
            .stdout.decode()
            .strip()
        )

        result["generate"] = True
        console.print(f"[bold][blue]Done 👍!!!")
        
        return result



import rclpy
import shutil
from rclpy.node import Node
from rclpy.serialization import serialize_message
import rosbag2_py
from rich.console import Console
from gazebo_msgs.msg import LinkStates
from std_msgs.msg import String
from rosgraph_msgs.msg import Clock
from action_msgs.msg import GoalStatusArray
from rclpy.qos import QoSProfile, QoSReliabilityPolicy, QoSHistoryPolicy
import tf2_ros
from tf2_msgs.msg import TFMessage



class SimpleBagRecorder(Node):
    def __init__(self):
        super().__init__('simple_bag_recorder')
        self.tf_buffer = tf2_ros.buffer.Buffer()
        self.r = tf2_ros.TransformBroadcaster(self)
        self.listener = tf2_ros.TransformListener(self.tf_buffer, self)
        self.writer = rosbag2_py.SequentialWriter()
        try:
            shutil.rmtree("my_bag")
        except:
            pass
        storage_options = rosbag2_py._storage.StorageOptions(
            uri='my_bag',
            storage_id='sqlite3')
        converter_options = rosbag2_py._storage.ConverterOptions('', '')

        self.writer.open(storage_options, converter_options)

        topic_info = rosbag2_py._storage.TopicMetadata(
            name='/_makeit/link_states',
            type= 'gazebo_msgs/msg/LinkStates',
            serialization_format='cdr')
        tf_topic_info = rosbag2_py._storage.TopicMetadata(
            name='/tf',
            type='tf2_msgs/msg/TFMessage',
            serialization_format='cdr')
        topic2_info = rosbag2_py._storage.TopicMetadata(
            name='/_sprunner',
            type= 'std_msgs/msg/String',
            serialization_format='cdr')
        
        topic3_info = rosbag2_py._storage.TopicMetadata(
            name='/clock',
            type= 'rosgraph_msgs/msg/Clock',
            serialization_format='cdr')
        topic4_info = rosbag2_py._storage.TopicMetadata(
            name='/move_action/_action/status',
            type= 'action_msgs/msg/GoalStatusArray',
            serialization_format='cdr')
        qos_profile = QoSProfile(
            reliability = QoSReliabilityPolicy.BEST_EFFORT,
            history = QoSHistoryPolicy.KEEP_LAST,
            depth = 10
        )
        
        self.writer.create_topic(topic_info)
        self.writer.create_topic(topic2_info)
        self.writer.create_topic(topic3_info)
        self.writer.create_topic(topic4_info)
        self.writer.create_topic(tf_topic_info)
        # self.create_timer(0.5, self.topic_callback)

        self.create_subscription(LinkStates, '/_makeit/link_states', self.topic_callback, 10)
        self.create_subscription(String, '/_sprunner', self.topic2_callback, 10)
        self.create_subscription(Clock, '/clock', self.topic3_callback, qos_profile)
        self.create_subscription(GoalStatusArray,'/move_action/_action/status',self.topic5_callback,10)
        self.subscription = self.create_subscription(TFMessage,'/tf',self.topic4_callback,10)
        self.get_logger().info("\033[92m DATA COLLECTION STARTED \033[00m", once=True)

        self.current_gazebo_time = 0
       
    def topic_callback(self, data):
        self.writer.write('/_makeit/link_states', serialize_message(data), self.current_gazebo_time)
    
    def topic2_callback(self, data):
        self.writer.write('/_sprunner', serialize_message(data), self.current_gazebo_time)

    def topic3_callback(self, data):
        self.current_gazebo_time = data.clock.sec
    def topic4_callback(self, data):
        self.writer.write('/tf',serialize_message(data),self.current_gazebo_time)
    def topic5_callback(self, data):
        self.writer.write('/move_action/_action/status',serialize_message(data),self.current_gazebo_time)
        # print(len(data.status_list))
def evaluate():
    result = {}
    console = Console()
    rclpy.init()
    sbr = SimpleBagRecorder()
    try:
        rclpy.spin(sbr)
    except KeyboardInterrupt:
        console.print(f"[green]\nExiting Gracefully[/green]")
    # rclpy.shutdown()
    result["generate"] = False
    return result

if __name__ == '__main__':
    evaluate()






'''
To evaluate Task 2B of CL eYRC23-24
Name: Ravikumar C.
Credit: e-Yantra, IIT Bombay
CDate: 10-10-2023
'''

import rclpy
import shutil
from rclpy.node import Node
from rclpy.serialization import serialize_message
import rosbag2_py
import tf2_ros
from rich.console import Console
from nav_msgs.msg import Odometry
from geometry_msgs.msg import PoseStamped
from gazebo_msgs.msg import ModelStates
from std_msgs.msg import String


class SimpleBagRecorder(Node):
    def __init__(self):
        super().__init__('simple_bag_recorder_task_2b')
        self.tf_buffer = tf2_ros.buffer.Buffer()
        self.r = tf2_ros.TransformBroadcaster(self)
        self.listener = tf2_ros.TransformListener(self.tf_buffer, self)
        self.writer = rosbag2_py.SequentialWriter()
        try:
            shutil.rmtree("my_bag")
        except:
            pass
        storage_options = rosbag2_py._storage.StorageOptions(
            uri='my_bag',
            storage_id='sqlite3')
        converter_options = rosbag2_py._storage.ConverterOptions('', '')
        self.writer.open(storage_options, converter_options)

        topic_info_odom = rosbag2_py._storage.TopicMetadata(
            name='odom',
            type= 'geometry_msgs/msg/PoseStamped', #'std_msgs/msg/String',
            serialization_format='cdr')
        self.writer.create_topic(topic_info_odom)
        topic_info_model = rosbag2_py._storage.TopicMetadata(
            name='_model_tf',
            type= 'gazebo_msgs/msg/ModelStates', #'std_msgs/msg/String',
            serialization_format='cdr')
        self.writer.create_topic(topic_info_model)
        topic_info_timer_state = rosbag2_py._storage.TopicMetadata(
            name='_timer_state',
            type= 'std_msgs/msg/String', #'std_msgs/msg/String',
            serialization_format='cdr')
        self.writer.create_topic(topic_info_timer_state)

        self.odom_sub = self.create_subscription(Odometry, 'odom', self.topic1, 10)
        self.model_sub = self.create_subscription(ModelStates, '/_demo/model_states_demo', self.topic2, 10)
        self.create_timer(0.1, self.topic3)

        self.get_logger().info("\033[92m DATA COLLECTION STARTED FOR TASK 2B \033[00m", once=True)


    def topic1(self, msg):
        self.odom_raw=msg
        self.odom_msg=PoseStamped()
        self.odom_msg.header=self.odom_raw.header
        self.odom_msg.pose=self.odom_raw.pose.pose
        self.writer.write('odom', serialize_message(self.odom_msg), self.get_clock().now().nanoseconds)

    def topic2(self, msg):
        self.writer.write('_model_tf', serialize_message(msg), self.get_clock().now().nanoseconds)
        

    def topic3(self):
        publisher_names = self.get_publishers_info_by_topic("/cmd_vel")
        pub_str=""
        if publisher_names:
            for publisher_name in publisher_names:
                pub_str=pub_str+publisher_name.node_name+", "
        pub_str_msg=String()
        pub_str_msg.data=pub_str
        self.writer.write('_timer_state', serialize_message(pub_str_msg), self.get_clock().now().nanoseconds)
 


def evaluate():
    result = {}
    console = Console()
    rclpy.init()
    sbr = SimpleBagRecorder()
    try:
        rclpy.spin(sbr)
    except KeyboardInterrupt or RuntimeError: 
        console.print(f"[green]\nExiting Gracefully[/green]")
    # rclpy.shutdown()
    result["generate"] = False
    return result



"""
evaluator.py

This file is used as an utility in task 2B of FPGA theme.
It generates the dump_txt.txt file which is used in the simulation.

"""
# standard imports
import os
import heapq

# graph map for our theme
graph_map = {
    "0":  [ "1", "6", "10"],
    "1":  [ "0",  "2", "11"],
    "2":  [ "1",  "3",  "4", "5"],
    "3":  [ "2"],
    "4":  [ "2"],
    "5":  [ "2"],
    "6":  [ "0",  "7", "8", "9"],
    "7":  [ "6"],
    "8":  [ "6"],
    "9":  [ "6"],
    "10": [ "0", "11", "24", "26"],
    "11": [ "1", "10", "12", "19"],
    "12": ["11", "13", "14"],
    "13": ["12"],
    "14": ["12", "15", "16"],
    "15": ["14"],
    "16": ["14", "17", "18"],
    "17": ["16"],
    "18": ["16", "19", "21"],
    "19": ["11", "18", "20"],
    "20": ["19"],
    "21": ["18", "22", "23"],
    "22": ["21"],
    "23": ["21", "24", "30"],
    "24": ["10", "23", "25"],
    "25": ["24"],
    "26": ["10", "27", "28"],
    "27": ["26"],
    "28": ["26", "29", "30"],
    "29": ["28"],
    "30": ["23", "28", "31"],
    "31": ["30"]
}

# Dijkstra Algorithm
def dijkstra(graph, start, end):
    # Initialize distances and predecessors
    distances = {node: float('inf') for node in graph}
    predecessors = {node: None for node in graph}
    distances[start] = 0

    # Priority queue for nodes to visit
    priority_queue = [(0, start)]

    while priority_queue:
        current_distance, current_node = heapq.heappop(priority_queue)

        if current_distance > distances[current_node]:
            continue

        for neighbor in graph[current_node]:
            distance = current_distance + 1  # Assuming all edges have equal weight of 1

            if distance < distances[neighbor]:
                distances[neighbor] = distance
                predecessors[neighbor] = current_node
                heapq.heappush(priority_queue, (distance, neighbor))

    # Reconstruct the path from end to start
    path = []
    while end is not None:
        path.insert(0, end)
        end = predecessors[end]

    return path


# Find all the paths connected between two points
def find_all_paths(graph, start, end):

    # Find path using recursion
    def recursive_find_paths(current, path):
        if current == end:
            paths.append(path)
        else:
            for neighbor in graph[current]:
                if neighbor not in path:
                    recursive_find_paths(neighbor, path + [neighbor])

    paths = []
    recursive_find_paths(start, [start])

    return paths

# Function Evaluate
def evaluate():

    # Set path to generate dump file
    directory_path = "/simulation/modelsim/"

    try: # Exception Handling
        if os.path.isdir(os.getcwd() + directory_path):
            # Read user input
            SP = input("Enter Start Point (0 <= SP <= 31): ")
            EP = input("Enter End Point   (0 <= EP <= 31): ")
            print()

            if SP in graph_map and EP in graph_map:
                # Find the shortest path
                shortest_path = dijkstra(graph_map, SP, EP)

                if not shortest_path:
                    print("No path found between SP and EP.")
                else:
                    # Find all possible paths connecting two points
                    all_paths = find_all_paths(graph_map, SP, EP)

                    i = 1
                    if all_paths:
                        all_paths.insert(0, shortest_path)
                        for path in all_paths:
                            print("Path " + str(i)+ " -> " + str(path))
                            i += 1

                    print("\nChoose the path same as the path given by your t2b_path_planner.c.")

                    print("Path Number ranges from 1 to " + str(i-1))
                    path_no = int(input("Enter the path Number : "))

                    if 0 < path_no < i:
                        print("Path selected is " + str(all_paths[path_no-1]))
                        with open('simulation/modelsim/dump_txt.txt', 'w') as f:
                            for i in all_paths[path_no-1]:
                                f.write(i + '\n')
                        print("File dump_txt.txt has been created.")
                    else:
                        print("Enter Valid Path Number")

            else:
                print("Invalid start or end point!")

        else:
            print(f"The directory '{directory_path}' does not exist.")

    except Exception as e:
        print(f"An error occurred: {e}")

    # Do not generate JSON file
    result = {}
    result["generate"] = False
    return result



# standard imports
import platform
import subprocess
import sys
import os

# third-party imprts
import distro
from eyantra_autoeval.utils.common import is_docker, run_shell_command
from rich import print
from rich.console import Console
from rich.prompt import Confirm


def evaluate():
    result = {}
    console = Console()
    with console.status("[bold green]Gathering data...") as status:
        console.print(f"[green]Gathering system information[/green]")
        distribution = {
            "machine": platform.machine(),
            "release": platform.release(),
            "system": platform.system(),
            "version": distro.version(best=True),
            "name": distro.name(),
        }
        result["distro"] = distribution

        result["virtualized"] = "hypervisor" in run_shell_command("cat /proc/cpuinfo")

        result["dockerized"] = is_docker()

    self_os_declaration = False
    result["self_os_declaration_confirm"] = None
    
    if (
        result["distro"]["name"] != "Ubuntu"
        or "22.04" not in result["distro"]["version"]
        or result["distro"]["machine"] != "x86_64"
        or result["virtualized"]
        or result["dockerized"]
    ):
        console.print(
            f"""You seem to be using [bold][blue]{result["distro"]["name"]} {result["distro"]["version"]} ({result["distro"]["machine"]}) on {"virtualized" if result["virtualized"] else ""}{"dockerized" if result["dockerized"] else ""} environment[/bold][/blue]. We only support [bold][blue]Ubuntu 22.04 on (x86_64) on baremetal[/bold][/blue]. You may continue to use the existing setup, and you shall not be penalized for the same. However, we’ll officially only support Ubuntu 22.04 as the OS of choice, and we won’t be to help you out with any installation related questions or package version incompatibility issues."""
        )
        self_os_declaration = Confirm.ask(
            "Do you accept to take ownership of your setup and not seek help from e-Yantra for the same?"
        )
        result["self_os_declaration_confirm"] = self_os_declaration
    result["self-os-declaration"] = self_os_declaration

    with console.status("[bold green]Gathering data...") as status:
        console.print(f"[green]Gathering information on system packages[/green]")
        result["apt"] = {}
        result["apt"]["packages"] = {}
        packages = (
            subprocess.run(
                "apt list --installed",
                capture_output=True,
                shell=True,
            )
            .stdout.decode()
            .split("\n")
        )
        packages = packages[1:-1]
        packages = [p.split()[0:2] for p in packages]
        result["apt"]["packages"]["installed"] = packages

        console.print(f"[green]Gathering information on pip packages[/green]")
        packages = run_shell_command("pip3 freeze").split("\n")
        packages = [p.split("==") for p in packages]

        result["pip3"] = {}
        result["pip3"]["packages"] = {}
        result["pip3"]["packages"]["installed"] = packages

        console.print(f"[green]Gathering information on ROS version[/green]")
        result["ros"] = {}
        # result["ros"]["version"] = run_shell_command("ROS_DISTRO")
        result["ros"]["version"] = (
            subprocess.run(
                "printenv ROS_DISTRO",
                capture_output=True,
                shell=True,
            )
            .stdout.decode()
            .strip()
        )
        # result["ros"]["packages"] = {}
        # packages = run_shell_command("rosversion -a").split("\n")
        # packages = [p.split(": ") for p in packages]
        # result["ros"]["packages"]["installed"] = packages

        console.print(f"[green]Gathering information on gazebo[/green]")
        result["gazebo"] = {}
        result["gazebo"]["version"] = (
            subprocess.run(
                "ign gazebo --version | grep 'Gazebo' | awk '{print $NF}'",
                capture_output=True,
                shell=True,
            )
            .stdout.decode()
            .strip()
        )

        console.print(f"[green]Gathering information on python[/green]")
        result["python"] = {}
        result["python"]["version"] = (
            subprocess.run(
                "python3 -V",
                capture_output=True,
                shell=True,
            )
            .stdout.decode()
            .strip()
            .split()[1]
        )

        console.print(f"[green]Gathering information on ros_gz_bridge[/green]")
        result["ros_gz_bridge"] = {}
        result["ros_gz_bridge"] = (
            subprocess.run(
                "ros2 pkg prefix ros_gz_bridge",
                capture_output=True,
                shell=True,
            )
            .stdout.decode()
            .strip()
        )

        if result["ros_gz_bridge"] != "/opt/ros/humble":
            result["generate"] = False
            console.print(f"[bold][red]ros_gz_bridge is not found on this system.")
        else:
            console.print(f"[bold][green]ros_gz_bridge is succesfully installed on this system.")

        result["generate"] = True
        if result["ros"]["version"] != "humble":
            result["generate"] = False
            console.print(f"[bold][red]ROS2 humble is not installed on this system.")
        else:
            console.print(f"[bold][green]ROS2 humble is succesfully installed on this system.")

        if result["gazebo"]["version"] != "6.17.0":
            result["generate"] = False
            console.print(f"[bold][red]Gazebo fortress not found on this system.")
        else:
            console.print(f"[bold][green]Gazebo fortress is succesfully installed on this system.")

        if result["self-os-declaration"] == False:
            if result["self_os_declaration_confirm"] == False:
                result["generate"] = False
                console.print(f"[bold][red]You have not agreed to take ownership for unsupported OS.")
        # Inform participants about the problem with installations
        if not result["generate"]:
            console.print(f"[bold][yellow]Detected Problems with Installations!")
            console.print(f"[bold][red]JSON file is not generated.")
            
        if result["generate"] == True:
            console.print(f"[bold][blue]You are all set to Fly!")


    return result



"""
evaluator.py

This file is used to evaluate task 2A of AstroTinker Bot theme.
It generates the dump_txt.txt file which is used in the simulation.

"""
import os

def toBinary(a):
    l,m=[],[]
    for i in a:
        l.append(ord(i))
    for i in l:
        m.append(bin(i)[2:].zfill(8))
    return m


def evaluate():

    directory_path = "/simulation/modelsim/"
    result = {}

    try:
        if os.path.isdir(os.getcwd() + directory_path):
            text = input("Enter the text: ")

            if len(text) <= 10:
                data = toBinary(text)
                with open('simulation/modelsim/dump_txt.txt', 'w') as f:
                    for i in data:
                        f.write(str(0))
                        f.write('\n')
                        for j in i:
                            f.write(j)
                            f.write('\n')
                        f.write(str(1))
                        f.write('\n')

                print("File dump_txt.txt has been created.")

            else:
                print("Entered text size is greater than 10!")
        else:
            print(f"The directory '{directory_path}' does not exist.")

    except Exception as e:
        print(f"An error occurred: {e}")

    result["generate"] = False
    return result